import json
import pytest
import re


@pytest.mark.prometheus
def test_prometheus_alerts(host):
    def summarize_alerts(alerts):
        """Print a alert summary."""
        print('Total alerts: %s' % len(alerts))
        print(json.dumps(alerts, indent=2))

    print("Starting prometheus test...")

    url = 'http://127.0.0.1:30090/api/v1/alerts'
    alert_json = json.loads(host.check_output('curl ' + url))
    status = alert_json["status"]
    alerts = alert_json["data"]["alerts"]

    real_alerts = []
    ignored_alerts = []

    for alert in alerts:
        # Filter out the ever firing "Dead mans switch" test alert
        if ignore_alert(alert):
            ignored_alerts.append(alert)
        else:
            real_alerts.append(alert)

    print('\n\n\n========= Ignored ==========')
    summarize_alerts(ignored_alerts)

    print('\n\n\n========= Firing ==========')
    summarize_alerts(real_alerts)

    count = len(real_alerts)

    assert status == "success", "Failure queriying the prometheus api at" + url
    assert count == 0, "Firing alerts: {0}".format(count)

def ignore_alert(alert):
    """
    Returns true if an alert should be ignored. This is when the "severity"
    equals "none", or in some application specific cases.
    """
    if alert["labels"]["severity"] == "none":
        return True
            # Ignore `KubeAPILatencyHigh` fom high load during installation
            # phase
    if alert["labels"]["alertname"] == "KubeAPILatencyHigh":
        return True
    # Filter out failing Nextcloud installation jobs since a lot of
    # them fail until they succeed during installation
    if "nextcloud" in alert["labels"]["pod"]:
        if alert["labels"]["alertname"] in [
                "KubeJobFailed",
                "KubeJobCompletion",
                "KubePodNotReady"]:
            return True
    # Filter out when RocketChat pods take very long to start
    if "rocketchat" in alert["labels"]["pod"]:
        if alert["labels"]["alertname"] in [
                "KubePodNotReady",
                "KubeDeploymentReplicasMismatch",
                "KubeStatefulSetReplicasMismatch"]:
            return True
    return False


if __name__ == "__main__":
    test_prometheus_alerts('')
